@@ -6,6 +6,7 @@ import os
 import re
 import posixpath
 import warnings
+import six
 from six import moves
 from w3lib.util import unicode_to_str
 
@@ -17,12 +18,30 @@ _ALWAYS_SAFE_BYTES = (b'ABCDEFGHIJKLMNOPQRSTUVWXYZ'
 
 
 def urljoin_rfc(base, ref, encoding='utf-8'):
-    """Same as urlparse.urljoin but supports unicode values in base and ref
+    r"""
+    .. warning::
+
+        This function is deprecated and will be removed in future.
+        Please use ``urlparse.urljoin`` instead.
+
+    Same as urlparse.urljoin but supports unicode values in base and ref
     parameters (in which case they will be converted to str using the given
     encoding).
 
     Always returns a str.
+
+    >>> import w3lib.url
+    >>> w3lib.url.urljoin_rfc('http://www.example.com/path/index.html', u'/otherpath/index2.html')
+    'http://www.example.com/otherpath/index2.html'
+    >>>
+
+    >>> w3lib.url.urljoin_rfc('http://www.example.com/path/index.html', u'fran\u00e7ais/d\u00e9part.htm')
+    'http://www.example.com/path/fran\xc3\xa7ais/d\xc3\xa9part.htm'
+    >>>
+
+
     """
+
     warnings.warn("w3lib.url.urljoin_rfc is deprecated, use urlparse.urljoin instead",
         DeprecationWarning)
 
@@ -76,7 +95,34 @@ def is_url(text):
     return text.partition("://")[0] in ('file', 'http', 'https')
 
 def url_query_parameter(url, parameter, default=None, keep_blank_values=0):
-    """Return the value of a url parameter, given the url and parameter name"""
+    """Return the value of a url parameter, given the url and parameter name
+
+    General case:
+
+    >>> import w3lib.url
+    >>> w3lib.url.url_query_parameter("product.html?id=200&foo=bar", "id")
+    '200'
+    >>>
+
+    Return a default value if the parameter is not found:
+
+    >>> w3lib.url.url_query_parameter("product.html?id=200&foo=bar", "notthere", "mydefault")
+    'mydefault'
+    >>>
+
+    Returns None if `keep_blank_values` not set or 0 (default):
+
+    >>> w3lib.url.url_query_parameter("product.html?id=", "id")
+    >>>
+
+    Returns an empty string if `keep_blank_values` set to 1:
+
+    >>> w3lib.url.url_query_parameter("product.html?id=", "id", keep_blank_values=1)
+    ''
+    >>>
+
+    """
+
     queryparams = moves.urllib.parse.parse_qs(
         moves.urllib.parse.urlsplit(str(url))[3],
         keep_blank_values=keep_blank_values
@@ -84,11 +130,33 @@ def url_query_parameter(url, parameter, default=None, keep_blank_values=0):
     return queryparams.get(parameter, [default])[0]
 
 def url_query_cleaner(url, parameterlist=(), sep='&', kvsep='=', remove=False, unique=True):
-    """Clean url arguments leaving only those passed in the parameterlist keeping order
+    """Clean URL arguments leaving only those passed in the parameterlist keeping order
+
+    >>> import w3lib.url
+    >>> w3lib.url.url_query_cleaner("product.html?id=200&foo=bar&name=wired", ('id',))
+    'product.html?id=200'
+    >>> w3lib.url.url_query_cleaner("product.html?id=200&foo=bar&name=wired", ['id', 'name'])
+    'product.html?id=200&name=wired'
+    >>>
+
+    If `unique` is ``False``, do not remove duplicated keys
+
+    >>> w3lib.url.url_query_cleaner("product.html?d=1&e=b&d=2&d=3&other=other", ['d'], unique=False)
+    'product.html?d=1&d=2&d=3'
+    >>>
+
+    If `remove` is ``True``, leave only those **not in parameterlist**.
+
+    >>> w3lib.url.url_query_cleaner("product.html?id=200&foo=bar&name=wired", ['id'], remove=True)
+    'product.html?foo=bar&name=wired'
+    >>> w3lib.url.url_query_cleaner("product.html?id=2&foo=bar&name=wired", ['id', 'foo'], remove=True)
+    'product.html?name=wired'
+    >>>
 
-    If remove is True, leave only those not in parameterlist.
-    If unique is False, do not remove duplicated keys
     """
+
+    if isinstance(parameterlist, (six.text_type, bytes)):
+        parameterlist = [parameterlist]
     url = moves.urllib.parse.urldefrag(url)[0]
     base, _, query = url.partition('?')
     seen = set()
@@ -106,24 +174,37 @@ def url_query_cleaner(url, parameterlist=(), sep='&', kvsep='=', remove=False, u
             seen.add(k)
     return '?'.join([base, sep.join(querylist)]) if querylist else base
 
-def add_or_replace_parameter(url, name, new_value, sep='&', url_is_quoted=False):
-    """Add or remove a parameter to a given url"""
-    def has_querystring(url):
-        _, _, _, query, _ = moves.urllib.parse.urlsplit(url)
-        return bool(query)
+def add_or_replace_parameter(url, name, new_value):
+    """Add or remove a parameter to a given url
 
-    parameter = url_query_parameter(url, name, keep_blank_values=1)
-    if url_is_quoted:
-        parameter = moves.urllib.parse.quote(parameter)
-    if parameter is None:
-        if has_querystring(url):
-            next_url = url + sep + name + '=' + new_value
+    >>> import w3lib.url
+    >>> w3lib.url.add_or_replace_parameter('http://www.example.com/index.php', 'arg', 'v')
+    'http://www.example.com/index.php?arg=v'
+    >>> w3lib.url.add_or_replace_parameter('http://www.example.com/index.php?arg1=v1&arg2=v2&arg3=v3', 'arg4', 'v4')
+    'http://www.example.com/index.php?arg1=v1&arg2=v2&arg3=v3&arg4=v4'
+    >>> w3lib.url.add_or_replace_parameter('http://www.example.com/index.php?arg1=v1&arg2=v2&arg3=v3', 'arg3', 'v3new')
+    'http://www.example.com/index.php?arg1=v1&arg2=v2&arg3=v3new'
+    >>>
+
+    """
+    parsed = moves.urllib.parse.urlsplit(url)
+    args = moves.urllib.parse.parse_qsl(parsed.query, keep_blank_values=True)
+
+    new_args = []
+    found = False
+    for name_, value_ in args:
+        if name_ == name:
+            new_args.append((name_, new_value))
+            found = True
         else:
-            next_url = url.rstrip('?') + '?' + name + '=' + new_value
-    else:
-        next_url = url.replace(name+'='+parameter,
-                               name+'='+new_value)
-    return next_url
+            new_args.append((name_, value_))
+
+    if not found:
+        new_args.append((name, new_value))
+
+    query = moves.urllib.parse.urlencode(new_args)
+    return moves.urllib.parse.urlunsplit(parsed._replace(query=query))
+
 
 def path_to_file_uri(path):
     """Convert local filesystem path to legal File URIs as described in:
