@@ -1,13 +1,15 @@
+import six
+
 def str_to_unicode(text, encoding=None, errors='strict'):
     if encoding is None:
         encoding = 'utf-8'
-    if isinstance(text, str):
+    if isinstance(text, bytes):
         return text.decode(encoding, errors)
     return text
 
 def unicode_to_str(text, encoding=None, errors='strict'):
     if encoding is None:
         encoding = 'utf-8'
-    if isinstance(text, unicode):
+    if isinstance(text, six.text_type):
         return text.encode(encoding, errors)
     return text
